<?php
/**
 * @file
 * Helper functions for the Conference module.
 */

/**
 * Returns an array of views, used to populate select lists.
 */
function _conference_get_views() {
  $views_list = views_get_all_views();
  ksort($views_list);

  $views = [];

  foreach ( $views_list as $name => $view ) {
    ksort($view->display);
    foreach ( $view->display as $display_name => $display ) {
      $views[$view->human_name . ' (' . $name . ')'][$name . ':' . $display_name] = $display->display_title . ' (' . $display_name . ')';
    }
  }

  return $views;
}

/**
 * Returns area nodes of which a certain user is chair.
 */
function _conference_get_user_areas($account = NULL, $full = FALSE) {
  $account = empty($account) ? $GLOBALS['user'] : $account;
  $area_chair_uid = is_object($account) ? $account->uid : $account;

  $user_areas = &drupal_static(__FUNCTION__);

  if (empty($user_areas[$area_chair_uid])) {
    $areas = (new EntityFieldQuery)
      ->entityCondition('entity_type', 'node')
      ->entityCondition('bundle', 'area')
      ->fieldCondition('conference_area_chair', 'target_id', $area_chair_uid)
      ->execute();
    $user_areas[$area_chair_uid] = reset($areas);
  }

  if ( !empty($user_areas[$area_chair_uid]) ) {
    if ( $full ) {
      return entity_load('node', array_keys($user_areas[$area_chair_uid]));
    }

    return $user_areas[$area_chair_uid];
  }

  return [];
}

/**
 * Checks whether user is area chair.
 *
 * @param $node
 *   A full object or nid of an area, presentation, or session node.
 * @param $account
 *   A full user object or uid.
 *
 * @todo Move somewhere more appropriate?
 * @todo make hookable?
 * @todo make static/cacheable?
 */
function _is_area_chair($node, $account = NULL) {
  $w = entity_metadata_wrapper('node', $node);

  try {
    $type = $w->type->value();
  }
  catch (EntityMetadataWrapperException $e) {
    return FALSE;
  }

  $account = !empty($account) ? $account : $GLOBALS['user'];
  switch ($type) {
    case 'area' :
      $conference_area_chair = $w->conference_area_chair->raw();
      if (!empty($conference_area_chair)) {
        if (in_array($account->uid, $conference_area_chair)) {
          return TRUE;
        }
      }
    break;

    case 'presentation' :
      $area = $w->conference_area->raw();
      return !empty($area) ? _is_area_chair($area, $account) : FALSE;
    break;

    case 'session' :
      $areas = $w->conference_areas->raw();
      if (!empty($areas)) {
        foreach ($areas as $nid ) {
          // Return TRUE as soon as a match is found
          return _is_area_chair($nid, $account);
        }
      }
    break;
  }
}

/**
 * Checks whether the node is in its grace period.
 */
function _is_grace_period($node) {
  $grace = REQUEST_TIME - $node->created;
  $grace_duration = cvget('conference_submissions_grace_period') * 3600;
  $conference = conference_get_conference($node);

  if ( $grace <= $grace_duration ) {
    if (conference_date_check('conference_submissions_period', 'open', $conference)) {
      return TRUE;
    }
  }
}

/**
 * Checks whether a node is frozen.
 *
 * @todo Rebuild using appropriate Flag API.
 */
function _is_node_frozen($node, $account = NULL) {
  $node = is_object($node) ? $node : node_load($node);

  if ( in_array($node->type, ['presentation', 'session']) ) {
    // The node is never frozen if the user has permission to override
    if ( user_access('override node freeze', $account) ) {
      return FALSE;
    }

    $nid = $node->nid;
    static $freeze;
    $freeze = empty($freeze) ? flag_get_flag('node_freeze') : $freeze;

    if ( $freeze->is_flagged($nid) ) {
      return TRUE;
    }

    // If node is not frozen, check for other freeze conditions
    $node = is_object($node) ? $node : node_load($node);
    $account = empty($account) ? $GLOBALS['user'] : $account;

    // if user is area chair
    if ( _is_area_chair($node, $account) ) {
      $conference = conference_get_conference($node);
      // Area chair operations haven't started (a negative deadline)
      // OR Area chair deadline is past
      if ($node->type == 'presentation' && conference_date_check('conference_submissions_approval', 'closed', $conference)) {
        return TRUE;
      }

      if ($node->type == 'session' && conference_date_check('conference_submissions_sessions', 'closed', $conference)) {
        return TRUE;
      }
    }
  }
}
