<?php
/**
 * @file
 * Registration workflow.
 */

/**
 * Menu callback: exhibitor-registration page.
 */
function conference_registration_exhibitor_page($data_id = NULL, $step = NULL) {
  // Add new exhibitor registration
  $data = new StoreFeatureExhibitorData($data_id);

  if ($data->is_new) {
    drupal_set_title(t('New exhibitor order'));
    $output['recipient'] = drupal_get_form('conference_registration_exhibitor_recipient', $data);
    // List pending registrations
    $output['pending'] = conference_registration_exhibitor_pending();
    return $output;
  }

  switch ($step) {
    case 'account' :
      drupal_set_title(t('Exhibitor order: account information'));
      return drupal_get_form('conference_registration_exhibitor_recipient_account', $data);
    break;

    case 'amount' :
      drupal_set_title(t('Exhibitor order: amount'));
      return drupal_get_form('conference_registration_exhibitor_recipient_amount', $data);
    break;

    case 'delete' :
      return drupal_get_form('conference_registration_exhibitor_delete_form', $data);
    break;
  }

  return !empty($output) ? $output : [];
}

/**
 * Form constructor: Collects exhibitor's email address.
 */
function conference_registration_exhibitor_recipient($form, &$form_state) {
  $data = $form_state['build_info']['args'][0];

  $form['recipient'] = [
    '#type' => 'emailfield',
    '#title' => t('Email address of the exhibitor you want to register'),
    '#required' => TRUE,
  ];

  $form['actions'] = ['#type' => 'actions'];
  $form['actions']['submit'] = [
    '#type' => 'submit',
    '#value' => t('Continue'),
  ];

  return $form;
}

/**
 * Submit handler for conference_registration_exhibitor_recipient().
 */
function conference_registration_exhibitor_recipient_submit($form, &$form_state) {
  $data = $form_state['build_info']['args'][0];
  $form_state['redirect'] = "admin/conference/registrations/exhibitor/{$data->id}";
  $data->set_recipient($form_state['values']['recipient'])->save();

  if (empty($data->recipient->uid)) {
    $form_state['redirect'] .= "/account";
  }
  else {
    $form_state['redirect'] .= "/amount";
  }
}

/**
 * Form constructor: Collects information to create a new user.
 */
function conference_registration_exhibitor_recipient_account($form, &$form_state) {
  $data = $form_state['build_info']['args'][0];

  $form['help'] = [
    '#markup' => '<p>' . t('The address %email does not correspond to an existing account. Enter the person’s information to create a new account.', ['%email' => $data->recipient->mail]) . '</p>',
  ];

  $form['recipient'] = [
    '#tree' => TRUE,
  ];

  $form['recipient']['firstname'] = [
    '#type' => 'textfield',
    '#title' => t('First name'),
  ];

  $form['recipient']['middlename'] = [
    '#type' => 'textfield',
    '#title' => t('Middle name'),
  ];

  $form['recipient']['lastname'] = [
    '#type' => 'textfield',
    '#title' => t('Last name'),
    '#required' => TRUE,
  ];

  $form['recipient']['legacy_affiliation'] = [
    '#type' => 'textfield',
    '#title' => t('Affiliation'),
    '#required' => TRUE,
  ];

  $form['actions'] = ['#type' => 'actions'];
  $form['actions']['submit'] = [
    '#type' => 'submit',
    '#value' => t('Create account and continue'),
  ];

  return $form;
}

/**
 * Submit handler for conference_registration_exhibitor_recipient_account().
 */
function conference_registration_exhibitor_recipient_account_submit($form, &$form_state) {
  $data = $form_state['build_info']['args'][0];
  $form_state['redirect'] = "admin/conference/registrations/exhibitor/{$data->id}/amount";

  // Save recipient account
  user_save($data->recipient);

  $profile = entity_create('profile2', [
    'type' => 'profile',
    'uid' => $data->recipient->uid,
  ]);

  foreach (array_filter($form_state['values']['recipient']) as $key => $value) {
    $profile->{"field_{$key}"} = [
      LANGUAGE_NONE => [[
        'value' => $value,
      ]],
    ];
  }

  profile2_save($profile);
  // Save account again to update user name
  user_save($data->recipient);
  $data->save();
}

/**
 * Form constructor: Collects exhibitor's email address.
 */
function conference_registration_exhibitor_recipient_amount($form, &$form_state) {
  $data = $form_state['build_info']['args'][0];

  $form['amount'] = [
    '#type' => 'textfield',
    '#title' => t('Order amount'),
    '#size' => 5,
    '#field_suffix' => $data->currency['code'],
    '#min' => 0,
    '#default_value' => $data->get_amount(),
    '#required' => TRUE,
  ];

  $form['actions'] = ['#type' => 'actions'];
  $form['actions']['submit'] = [
    '#type' => 'submit',
    '#value' => t('Create order'),
  ];

  return $form;
}

/**
 * Validation handler for conference_registration_exhibitor_recipient_amount().
 */
function conference_registration_exhibitor_recipient_amount_validate($form, &$form_state) {
  $data = $form_state['build_info']['args'][0];

  if ($data->calculate_amount($form_state['values']['amount']) <= 0) {
    form_set_error('amount', t('The amount must be larger than zero.'));
  }
}

/**
 * Submit handler for conference_registration_exhibitor_recipient_amount().
 */
function conference_registration_exhibitor_recipient_amount_submit($form, &$form_state) {
  $data = $form_state['build_info']['args'][0];
  $data->set_amount($form_state['values']['amount'])->save();
  $link = url("conference/exhibitor", ['absolute' => TRUE]);
  $replacements = [
    '!amount' => $data->currency['symbol'] . $data->get_amount(),
    '!buyer' => format_username(user_load($data->recipient)),
    '!link' => l($link, $link),
  ];
  drupal_set_message(t('You successfully created an exhibitor registration for !amount. Please direct !buyer to !link to complete the order.', $replacements));
  $form_state['redirect'] = 'admin/conference/registrations/exhibitor';
}

/**
 * Displays a list of pending exhibitor registrations.
 */
function conference_registration_exhibitor_pending() {
  $data = (array) cvget('conference_registration_exhibitor_form', ['data_id' => '%']);

  if (!empty($data['multiple'])) {
    $data = array_filter($data['multiple'], function($item) {
      return !empty($item['complete']) && !empty($item['recipient'] && !is_object($item['recipient']));
    });
  }

  if (!empty($data)) {
    array_walk($data, function(&$item) {
      $item['weight'] = $item['time'];
    });

    uasort($data, 'drupal_sort_weight');

    array_walk($data, function(&$item) {
      $item = new StoreFeatureExhibitorData($item);
    });

    $header = [
      'id' => t('ID'),
      'conference' => t('Conference'),
      'exhibitor' => t('Exhibitor'),
      'date' => t('Date'),
      'amount' => t('Amount'),
      'operations' => '',
    ];

    foreach ($data as $item) {
      $recipient = user_load($item->recipient);

      $rows[$item->id] = [
        'id' => $item->id,
        'conference' => conference_get_conference_year($item->conference),
        'exhibitor' => l(format_username($recipient), "user/{$recipient->uid}"),
        'date' => format_date($item->time, 'custom', 'c'),
        'amount' => $item->currency['symbol'] . $item->get_amount(),
        'operations' => l(t('Delete'), "admin/conference/registrations/exhibitor/{$item->id}/delete"),
      ];
    }

    return $output['pending'] = [
      'title' => [
        '#markup' => '<h3>' . t('Pending exhibitor orders') . '</h3>',
      ],
      'table' => [
        '#theme' => 'table',
        '#header' => $header,
        '#rows' => $rows,
      ],
    ];
  }
}

/**
 * Form constructor: Confirms deletion of exhibitor order's data.
 */
function conference_registration_exhibitor_delete_form($form, &$form_state) {
  $data = $form_state['build_info']['args'][0];

  $form['data'] = [
    '#type' => 'value',
    '#value' => $data,
  ];

  return confirm_form($form,
  t('Do you want delete this exhibitor order?'),
  'admin/conference/registrations/exhibitor',
  t('This action cannot be undone.'),
  t('Delete order'),
  t('Cancel'));
}

/**
 * Submit handler for conference_registration_exhibitor_delete_form().
 */
function conference_registration_exhibitor_delete_form_submit($form, &$form_state) {
  $data = $form_state['build_info']['args'][0];
  $data->reload();

  if (!$data->is_new) {
    $data->delete();
    drupal_set_message(t('Order deleted.'));
  }
}

/**
 * Menu callback: displays a list of exhibitor registrations for end users.
 */
function conference_registration_exhibitor_list($id = NULL) {
  global $user;

  $data = cvget('conference_registration_exhibitor_form', ['data_id' => '%']);
  $data = array_filter($data['multiple'], function($item) use ($user) {
    return !empty($item['complete']) && $item['recipient'] == $user->uid;
  });

  $header = [
    'id' => t('ID'),
    'conference' => t('Conference'),
    'date' => t('Date'),
    'amount' => t('Amount'),
    'operations' => '',
  ];

  $rows = [];

  if (!empty($data)) {
    array_walk($data, function(&$item) {
      $item['weight'] = $item['time'];
    });

    uasort($data, 'drupal_sort_weight');

    array_walk($data, function(&$item) {
      $item = new StoreFeatureExhibitorData($item);
    });

    foreach ($data as $item) {
      $recipient = user_load($item->recipient);

      $rows[$item->id] = [
        'id' => $item->id,
        'conference' => conference_get_conference_year($item->conference),
        // 'exhibitor' => l(format_username($recipient), "user/{$recipient->uid}"),
        'date' => format_date($item->time, 'custom', 'c'),
        'amount' => $item->currency['symbol'] . $item->get_amount(),
        'operations' => l(t('Add to cart'), "conference/exhibitor/{$item->id}"),
      ];
    }
  }

  return [
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
    '#empty' => t('You have no exhibitor orders'),
  ];
}

/**
 * Menu callback: exhibitor registration's add-to-cart confirmation form.
 */
function conference_registration_exhibitor_order_form($form, &$form_state, $data_id) {
  $form['data'] = [
    '#type' => 'value',
    '#value' => new StoreFeatureExhibitorData($data_id),
  ];

  return confirm_form($form,
  t('Do you want to add this exhibitor registration to your shopping cart?'),
  'conference/exhibitor',
  '',
  t('Add it to cart'),
  t('Cancel'));
}

/**
 * Submission handler for conference_registration_exhibitor_order_form().
 */
function conference_registration_exhibitor_order_form_submit($form, &$form_state) {
  $data = $form_state['values']['data'];
  $data->reload(); // Make sure it still exists
  $form_state['redirect'] = url('cart/conference/exhibitor');

  if (!$data->is_new) {

    $product = $data->get_product();

    if (!empty($product)) {
      $line_item = entity_create('commerce_line_item', [
        'type' => 'product',
        'data' => [
          'context' => [
            'conference_registration_type' => 'exhibitor',
            'amount' => $data->amount,
            'add_to_cart_combine' => FALSE,
            'configure' => $data->variable(),
          ],
        ],
      ]);

      module_load_include('inc', 'conference_registration', 'conference_registration.registration');
      conference_registration_add_to_cart($product, $line_item, TRUE);
      $data->delete();

      // Mark the cart as an exhibitor order (useful to properly redirect after deleting cart)
      $cart = commerce_cart_order_load($data->recipient);
      $cart->data['conference_registration_type'] = 'exhibitor';
      commerce_order_save($cart);
    }
  }
  else {
    drupal_set_message(t('The selected order is already in your cart.'), 'warning');
  }
}

/**
 * For fun, let's declare a supporting class here. It should have its own file,
 * but it won't be used anywhere else.
 */
class StoreFeatureExhibitorData {
  public $id;
  public $conference;
  public $recipient;
  public $time;
  public $amount = 0;
  public $currency;
  public $is_new = FALSE;
  public $complete = FALSE;
  public $product = NULL;

  public function __construct($id = NULL) {
    global $user;

    if (!empty($id)) {
      $data = is_array($id) ? $id : cvget('conference_registration_exhibitor_form', ['data_id' => $id]);
    }

    if (!empty($data)) {
      foreach ($data as $key => $value) {
        $this->{$key} = $value;
      }
    }
    else {
      $this->id = substr(sha1($user->uid . REQUEST_TIME . rand(0, microtime(TRUE))), 0, 10);
      $this->time = REQUEST_TIME;
      $this->conference = conference_current();
      $this->is_new = TRUE;
      $this->recipient = NULL;
    }

    $this->currency = commerce_currency_load();
  }

  public function set_recipient($recipient) {
    if (is_object($recipient)) {
      $this->recipient = $recipient;
    }
    else {
      $this->recipient = $this->find_user($recipient);
    }

    return $this;
  }

  public function find_user($email) {
    // Double-check again if the email already exists in the system
    if (module_exists('multiple_email')) {
      $multi = multiple_email_find_address($email);
      if (!empty($multi)) {
        return user_load($multi->uid);
      }
    }

    $account = user_load_by_mail($email);
    if (empty($account)) {
      $account = entity_create('user', [
        'name' => user_password(5),
        'pass' => user_password(10),
        'mail' => $email,
        'status' => 1,
      ]);

      // Do not save account here, and wait until user decides to save it
    }

    return $account;
  }

  public function calculate_amount($amount) {
    return commerce_currency_decimal_to_amount($amount, $this->currency['code']);;
  }

  public function set_amount($amount) {
    $this->amount = $this->calculate_amount($amount);
    $this->complete = TRUE;
    return $this;
  }

  public function get_amount() {
    return commerce_currency_amount_to_decimal($this->amount, $this->currency['code']);
  }

  public function get_product() {
    if (empty($this->product)) {
      $products = (new EntityFieldQuery)
        ->entityCondition('entity_type', 'commerce_product')
        ->entityCondition('bundle', 'exhibitor_registration')
        ->fieldCondition('conference_reference', 'target_id', $this->conference)
        ->execute();

      if (!empty($products['commerce_product'])) {
        $this->product = commerce_product_load(reset($products['commerce_product'])->product_id);
      }
    }

    return $this->product;
  }

  public function variable() {
    return [
      'id' => $this->id,
      'conference' => $this->conference,
      'recipient' => (is_object($this->recipient) && !empty($this->recipient->uid)) ? $this->recipient->uid : $this->recipient,
      'time' => $this->time,
      'complete' => $this->complete,
      'amount' => $this->amount,
    ];
  }

  public function reload() {
    $this->__construct($this->id);
    return $this;
  }

  public function save() {
    cvset('conference_registration_exhibitor_form', $this->variable(), ['data_id' => $this->id]);
  }

  public function delete() {
    cvdel('conference_registration_exhibitor_form', ['data_id' => $this->id]);
  }
}
