<?php
/**
 * @file
 *
 * Functions related to custom node forms.
 */

/**
 * Menu callback: replaces node_add().
 *
 * @see node_add()
 */
function conference_submissions_node_add($type, $node = NULL) {
  if (!empty($node) && ($node->type != $type)) {
    return MENU_NOT_FOUND;
  }

  if (empty($node)) {
    // $node is a string, i.e. a node type
    if (user_access("use simplified $node form")) {
      // Use standard form
      module_load_include('inc', 'node', 'node.pages');
      return node_add($type);
    }

    // Create node stub
    $node = (object) [
      'type' => $type,
      'title' => '',
      'uid' => $GLOBALS['user']->uid,
      'name' => $GLOBALS['user']->name,
      'is_new' => TRUE,
    ];
  }

  switch ($node->type) {
    case 'presentation' :
      if (empty($node->conference_submissions_hash)) {
        // If there's no hash, it's a new presentation
        drupal_set_title(t('Submit a proposal'));

        // Adds a notice at the top of the presentation form.
        if ( !user_access('override node quota') ) {
          $guidelines = cvget('conference_submissions_guidelines_node');

          if (!empty($guidelines)) {
            drupal_set_message(t('Have you read our !guidelines?', ['!guidelines' => l(t('submission guidelines'), 'node/' . $guidelines, ['attributes' => ['target' => '_blank']])]), 'question', FALSE);
          }
        }

        $form = drupal_get_form('conference_submissions_node_form', $node);

        if (isset($form['presenter_email'])) {
          $subtitle = t('Basic information');
        }
        elseif (isset($form['accounts'])) {
          $subtitle = t('Presenters’ personal details');
        }

        return [
          'subtitle' => [
            '#markup' => !empty($subtitle) ? "<h3>$subtitle</h3>" : '',
          ],
          'form' => $form,
        ];
      }

    default :
      module_load_include('inc', 'node', 'node.pages');
      return drupal_get_form($node->type . '_node_form', $node);
  }
}

/**
 * Form constructor replacement for node forms.
 */
function conference_submissions_node_form($form, &$form_state, $node) {
  switch ($node->type) {
    case 'presentation' :
      $form_state['form_page'] = empty($form_state['form_page']) ? 'basic_info' : $form_state['form_page'];

      switch ( $form_state['form_page'] ) {
        // Step 1
        case 'basic_info' :
          $types = conference_submissions_presentation_types(TRUE);

          if (count($types) > 1) {
            $options = conference_submissions_field_allowed_values(['field_name' => 'conference_presentation_type', 'use_in_form' => TRUE]);
            $default_value = conference_submissions_field_default_value(NULL, NULL, ['field_name' => 'conference_presentation_type'], NULL);
            $default_value = reset($default_value)['value'];

            $form['presentation_type'] = [
              '#type' => 'select',
              '#title' => t('Presentation type'),
              '#options' => $options,
              '#required' => TRUE,
              '#default_value' => $default_value,
            ];
          }
          else {
            $types = array_keys($types);
            $form['presentation_type'] = [
              '#type' => 'value',
              '#value' => reset($types),
            ];
          }

          $wrapper_id = drupal_html_id('presenter-email-add-more-wrapper');

          $form['presenter_email'] = [
            '#type' => 'item',
            '#title' => t('Presenters’ email addresses'),
            '#tree' => TRUE,
            '#prefix' => '<div id="' . $wrapper_id . '">',
            '#suffix' => '</div>',
          ];

          $form['presenter_email']['help'] = [
            '#markup' => t('Enter email addresses for all the presenters in your proposal.'),
            '#prefix' => '<p>',
            '#suffix' => '</p>',
          ];

          if (user_access('create users on submission')) {
            $form['presenter_email']['help']['#markup'] .= ' ' . t('If we can’t match them with existing accounts, we’ll ask you to provide their personal information and we’ll create new accounts for them.');
          }
          else {
            $form['presenter_email']['help']['#markup'] .= ' ' . t('Make sure all co-presenters have already created accounts on the website.');
          }

          $items_count = empty($form_state['field_state']['presenter_email']['items_count']) ? 1 : $form_state['field_state']['presenter_email']['items_count'];

          for ( $i = 1; $i <= $items_count; $i++ ) {
            $form['presenter_email']['elements'][] = [
              '#field_name' => 'presenter_email',
              '#type' => 'emailfield',
            ];
          }

          if ( $items_count == 1 ) {
            $account = $GLOBALS['user'];
            $form['presenter_email']['elements'][0]['#default_value'] = $account->mail;

            // If only one presentation type is set, check quota immediately
            if (count($types) == 1) {
              $type = reset($types);
              if ($count = _user_is_over_quota($type, $account)) {
                drupal_set_message(t('You already have !presentation_count in the system. You can only add other people to this presentation.', ['!presentation_count' => format_plural($count, 'one presentation', '@count presentations')]), 'warning');
                $form['presenter_email']['elements'][0]['#default_value'] = '';
              }
            }
          }

          $form['presenter_email']['add_more'] = [
            '#type' => 'submit',
            '#value' => t('Add another presenter’s email'),
            '#submit' => ['conference_submissions_add_more_submit'],
            '#validate' => [],
            '#limit_validation_errors' => [],
            '#ajax' => [
              'callback' => 'conference_submissions_add_more_js',
              'wrapper' => $wrapper_id,
              'effect' => 'fade',
            ],
          ];

          $field_state = [
            'items_count' => count(element_children($form['presenter_email']['elements'])),
            'array_parents' => [],
            'errors' => [],
          ];

          $form_state['field_state']['presenter_email']['items_count'] = count(element_children($form['presenter_email']['elements']));

          $submit_value = t('Continue');
        break;

        // Step 2
        case 'account_details' :
          if ( !empty($form_state['submission']['presenters']['existing']) ) {
            foreach ( $form_state['submission']['presenters']['existing'] as $uid => $existing ) {
              $items[] = format_username($existing) . " ({$existing->mail})";
            }

            $form['accounts']['existing'] = [
              '#markup' => '<p>' . t('We found !existing_count:', ['!existing_count' => format_plural(count($form_state['submission']['presenters']['existing']), 'one existing account', '@count existing accounts')]) . '</p>' . theme('item_list', ['title' => NULL, 'type' => 'ul', 'items' => $items]),
            ];
          }

          if ( !empty($form_state['submission']['presenters']['new']) ) {
            $form['accounts']['new'] = [
              '#markup' => '<p>' . t('We created !new accounts:', ['!new accounts' => format_plural(count($form_state['submission']['presenters']['new']), 'one new account', '@count new accounts')]) . '</p>',
            ];

            $form['new_accounts'] = [
              '#type' => 'container',
              '#tree' => TRUE,
            ];

            foreach ( $form_state['submission']['presenters']['new'] as $uid => $new ) {
              // Call hooks to build user profiles

              $elements = module_invoke_all('conference_submissions_node_form_account_details');

              if (!empty($elements)) {
                $form['new_accounts'][$uid] = [
                  '#type' => 'container',
                  '#tree' => TRUE,
                ];

                $form['new_accounts'][$uid] = [
                  '#type' => 'fieldset',
                  '#title' => t('Details for !email', ['!email' => $new->mail]),
                ];

                foreach ($elements as $name => $element) {
                  $form['new_accounts'][$uid][$name] = $element;
                }
              }
            }
          }

          $submit_value = t('Continue');
        break;
      }
    break;
  }

  $form['submit'] = [
    '#type' => 'submit',
    '#value' => $submit_value,
  ];

  return $form;
}

/**
 * Validation handler for conference_submissions_node_form().
 */
function conference_submissions_node_form_validate($form, &$form_state) {
  switch ( $form_state['form_page'] ) {
    case 'basic_info' :
      // Clean up form values
      $state = $form_state;
      form_state_values_clean($state);
      $form_state['submission'] = $state['values'];
      $form_state['submission']['presenters'] = [];
      $emails = array_filter($form_state['submission']['presenter_email']['elements']);
      $type = $form_state['submission']['presentation_type'] = $form_state['submission']['presentation_type'];
      $type_name = conference_submissions_presentation_types()[$type]['name'];

      if ( empty($emails) ) {
        form_set_error("presenter_email][elements][0", t('You must enter at least one email address to proceed.'));
      }
      else {
        $new = FALSE;
        $error = FALSE;
        foreach ( $emails as $delta => $email ) {
          $user = _conference_submissions_get_user($email, $new, $error);

          if ($error) {
            $errors[] = $email;
            form_set_error("presenter_email][elements][$delta");
          }

          if ( $new ) {
            $form_state['submission']['presenters']['new'][$user->uid] = $user;
          }
          else {
            if ($count = _user_is_over_quota($type, $user)) {
              form_set_error("presenter_email][elements][$delta", t('!user already has !presentation_count of type %type_name in the system and cannot be added to a new one.', ['!user' => format_username($user) . " ($email)", '!presentation_count' => format_plural($count, 'one presentation', '@count presentations'), '%type_name' => $type_name]));
            }
            else {
              if (!empty($user)) {
                $form_state['submission']['presenters']['existing'][$user->uid] = $user;
              }
            }
          }
        }

        if (!empty($errors)) {
          $emails = implode(', ', $errors);
          $singular = 'We couldn’t find an account linked to the address %email. Make sure you entered it correctly.';
          $plural = 'We couldn’t find any accounts linked to these addresses: %email. Make sure you entered them correctly.';
          $reminder = t('Each co-presenter must create their own account before you can add them to a proposal.');
          drupal_set_message(format_plural(count($errors), $singular, $plural, ['%email' => $emails]) . ' ' . $reminder, 'error');
        }
      }
      unset($form_state['submission']['presenter_email']);
    break;
  }
}

/**
 * Submit handler for conference_submissions_node_form().
 */
function conference_submissions_node_form_submit($form, &$form_state) {
  $node = $form_state['build_info']['args'][0];

  switch ( $form_state['form_page'] ) {
    case 'basic_info' :
      if ( !empty($form_state['submission']['presenters']['new']) ) {
        $form_state['form_page'] = 'account_details';
        $form_state['rebuild'] = TRUE;
      }
    break;

    case 'account_details' :
      $new = $form_state['submission']['presenters']['new'];
      $existing = empty($form_state['submission']['presenters']['existing']) ? [] : $form_state['submission']['presenters']['existing'];

      foreach ( $form_state['values']['new_accounts'] as $uid => $data ) {
        $account = $form_state['submission']['presenters']['new'][$uid];
        module_invoke_all('conference_submissions_node_form_account_details_submit', $account, $data);
      }

      $form_state['submission']['presenters']['existing'] = $existing + $new;
    break;
  }

  if ( !$form_state['rebuild'] && !empty($form_state['submission']['presenters']['existing']) ) {
    global $user;

    if (!empty($form_state['submission']['presentation_type'])) {
      $node->conference_presentation_type[LANGUAGE_NONE][0]['value'] = $form_state['submission']['presentation_type'];
    }

    if (!empty($form_state['submission']['presenters']['existing'])) {
      foreach ($form_state['submission']['presenters']['existing'] as $uid => $account) {
        $node->conference_presenters[LANGUAGE_NONE][]['target_id'] = $uid;
      }
    }

    // Let other modules take care of any modifications they've made to the form
    drupal_alter('conference_submissions_node_form_submit', $node, $form_state['submission']);
    $hash = substr(drupal_hash_base64(serialize($node) . time()), 0, 6);
    $node->conference_submissions_hash = $hash;
    $node->conference_submissions_form = 'conference_submissions_presentation';

    $form_data = [
      'uid' => $user->uid,
      'created' => time(),
      'node' => $node,
    ];

    cvset('conference_submissions_node_form', $form_data, ['form_id' => $hash]);
    $form_state['redirect'] = "node/add/presentation/$hash";
  }
}

/**
 * Submit handler for the "Add another item" button of a field form.
 *
 * This handler is run regardless of whether JS is enabled or not. It makes
 * changes to the form state. If the button was clicked with JS disabled, then
 * the page is reloaded with the complete rebuilt form. If the button was
 * clicked with JS enabled, then ajax_form_callback() calls
 * conference_submissions_add_more_js() to return just the changed part
 * of the form.
 *
 * @see field_add_more_submit()
 */
function conference_submissions_add_more_submit($form, &$form_state) {
  $button = $form_state['triggering_element'];
  // Go one level up in the form, to the widgets container.
  $element = drupal_array_get_nested_value($form, array_slice($button['#array_parents'], 0, -1));
  // Increment the items count.
  $field_state = &$form_state['field_state'][reset($element['#array_parents'])];
  $field_state['items_count']++;
  $form_state['rebuild'] = TRUE;
}

/**
 * Ajax callback in response to a new empty widget being added to the form.
 *
 * This returns the new page content to replace the page content made obsolete
 * by the form submission.
 *
 * @see conference_submissions_add_more_submit()
 * @see field_add_more_js()
 */
function conference_submissions_add_more_js($form, $form_state) {
  $button = $form_state['triggering_element'];
  // Go one level up in the form, to the widgets container.
  $element = drupal_array_get_nested_value($form, array_slice($button['#array_parents'], 0, -1));
  $field_state = &$form_state['field_state'][reset($element['#array_parents'])];
  // Add a DIV around the delta receiving the Ajax effect.
  $delta = count(element_children($element['elements']));
  $element[$delta]['#prefix'] = '<div class="ajax-new-content">' . (isset($element[$delta]['#prefix']) ? $element[$delta]['#prefix'] : '');
  $element[$delta]['#suffix'] = (isset($element[$delta]['#suffix']) ? $element[$delta]['#suffix'] : '') . '</div>';

  return $element;
}

/**
 * Finds user given its e-mail address. If no user is found, one is created.
 * The $edit array must be provided to build a new user.
 */
function _conference_submissions_get_user($email, &$new = FALSE, &$error = FALSE) {
  // If multiple_email is enabled, use the relevant table, or fall back to the user table
  if ( module_exists('multiple_email') ) {
    $exists = multiple_email_find_address($email);

    if ( !empty($exists) ) {
      $user = user_load($exists->uid);
    }
  }
  else {
    $user = user_load_by_mail($email);
  }

  if ( empty($user) ) {
    if (user_access('create users on submission')) {
      $user = _conference_submissions_create_user($email);
      $new = TRUE;
      return $user;
    }

    $error = TRUE;
    return;
  }

  $new = FALSE;
  $error = FALSE;

  return $user;
}

/**
 * Creates a user and returns the $user object.
 *
 * @todo Make sure username generation works.
 */
function _conference_submissions_create_user($email) {
  $user = (object) [
    'name' => 'email_registration_' . user_password(), // This is to trigger email_registration's name creation
    'mail' => $email,
    'created' => REQUEST_TIME,
    'access' => 0,
    'login' => 0,
    'status' => 1,
    'language' => NULL,
    'init' => $email,
  ];

  // Create user if not existing
  // Not using drupal_write_record: user_save will auto-increment uid
  user_save($user, ['pass' => user_password()]);

  _user_mail_notify('conference_submissions_auto_user_created', $user);

  return $user;
}
